'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Track = function (_React$Component) {
    inherits(Track, _React$Component);

    function Track(props) {
        classCallCheck(this, Track);

        var _this = possibleConstructorReturn(this, (Track.__proto__ || Object.getPrototypeOf(Track)).call(this, props));

        _this.state = {
            isPlaying: false
        };
        _this.audio = new Audio();
        _this.getDuration = _this.getDuration.bind(_this);
        _this.playTrack = _this.playTrack.bind(_this);
        return _this;
    }

    createClass(Track, [{
        key: 'getDuration',
        value: function getDuration(ms) {
            var min = ms / 1000 / 60 << 0;
            var sec = ms / 1000 % 60;
            return Math.round(min, -2) + ':' + (Math.round(sec, -2) < 10 ? '0' + Math.round(sec, -2) : Math.round(sec, -2));
        }
    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.audio.setAttribute('src', this.props.track.preview_url);
            this.audio.load();
        }
    }, {
        key: 'componentWillUpdate',
        value: function componentWillUpdate(newProps) {
            if (newProps.track.preview_url !== this.props.track.preview_url) {
                this.audio.setAttribute('src', newProps.track.preview_url);
                this.audio.load();
            }
        }
    }, {
        key: 'playTrack',
        value: function playTrack(event) {
            event.preventDefault();
            if (this.audio.paused) {
                this.setState({ isPlaying: true }, function () {
                    this.audio.play();
                    this.props.onPlayingStatusChange(true, this.audio, this.props.track);
                    this.audio.addEventListener('ended', function () {
                        this.setState({ isPlaying: false });
                        this.props.onPlayingStatusChange(false, this.audio, this.props.track);
                    });
                });
            } else {
                this.audio.pause();
                this.setState({ isPlaying: false }, function () {
                    this.props.onPlayingStatusChange(false, this.audio, this.props.track);
                });
            }
        }
    }, {
        key: 'render',
        value: function render() {
            var playButtonClassNames = this.state.isPlaying ? 'fa fa-pause' : 'fa fa-play';
            var playButtonStyle = { cursor: 'default' };
            return React.createElement(
                'li',
                { className: this.props.listGroupItemClassName, onClick: this.playTrack },
                React.createElement(
                    'a',
                    { href: '#',
                        className: this.props.listGroupItemLink,
                        style: playButtonStyle,
                        title: this.props.tooltip
                    },
                    this.props.track.track_number,
                    '. ',
                    this.props.track.name,
                    ' (',
                    this.getDuration(this.props.track.duration_ms),
                    ')'
                ),
                React.createElement(
                    'span',
                    { className: this.props.listGroupItemBadgeClassName },
                    React.createElement('i', {
                        className: playButtonClassNames
                    })
                )
            );
        }
    }]);
    return Track;
}(React.Component);

Track.propTypes = {
    listGroupItemBadgeClassName: PropTypes.string,
    listGroupItemClassName: PropTypes.string,
    listGroupItemLink: PropTypes.string,
    onPlayingStatusChange: PropTypes.func, //handler on playing status change, function(isPlaying, audioTrack, spotifyTrack),
    tooltip: PropTypes.string.isRequired,
    track: PropTypes.object.isRequired
};

Track.defaultProps = {
    onPlayingStatusChange: undefined
};

var TrackList = function (_React$Component) {
    inherits(TrackList, _React$Component);

    function TrackList(props) {
        classCallCheck(this, TrackList);

        var _this = possibleConstructorReturn(this, (TrackList.__proto__ || Object.getPrototypeOf(TrackList)).call(this, props));

        _this.trackChangedStatus = _this.trackChangedStatus.bind(_this);
        return _this;
    }

    createClass(TrackList, [{
        key: 'trackChangedStatus',
        value: function trackChangedStatus(isPlaying, audioTrack, spotifyTrack) {
            this.props.updateTrackPlayingStatus(isPlaying, audioTrack, spotifyTrack);
        }
    }, {
        key: 'render',
        value: function render() {
            var trackNodesData = this.props.tracks.map(function (track, index) {
                return React.createElement(Track, {
                    key: index,
                    listGroupItemBadgeClassName: this.props.listGroupItemBadgeClassName,
                    listGroupItemClassName: this.props.listGroupItemClassName,
                    listGroupItemLink: this.props.listGroupItemLink,
                    onPlayingStatusChange: this.trackChangedStatus,
                    tooltip: this.props.previewWarningText,
                    track: track
                });
            }.bind(this));
            return React.createElement(
                'ul',
                { className: this.props.listGroupClassName },
                trackNodesData
            );
        }
    }]);
    return TrackList;
}(React.Component);

TrackList.propTypes = {
    listGroupItemBadgeClassName: PropTypes.string,
    listGroupClassName: PropTypes.string,
    listGroupItemClassName: PropTypes.string,
    listGroupItemLink: PropTypes.string,
    previewWarningText: PropTypes.string,
    tracks: PropTypes.array,
    updateTrackPlayingStatus: PropTypes.func
};

TrackList.defaultProps = {
    tracks: [],
    updateTrackPlayingStatus: undefined
};

var SpotifyPlayer = function (_React$Component) {
    inherits(SpotifyPlayer, _React$Component);

    function SpotifyPlayer(props) {
        classCallCheck(this, SpotifyPlayer);

        var _this = possibleConstructorReturn(this, (SpotifyPlayer.__proto__ || Object.getPrototypeOf(SpotifyPlayer)).call(this, props));

        _this.state = {
            albumData: null,
            playerPlaceholder: '<i class="fa fa-spinner fa-spin"></i>',
            albumName: _this.props.albumName,
            artistName: _this.props.artistName
        };
        _this.fetchAlbumData = _this.fetchAlbumData.bind(_this);
        _this.fetchTracks = _this.fetchTracks.bind(_this);
        _this.firePlayingStatusChangeEvent = _this.firePlayingStatusChangeEvent.bind(_this);
        return _this;
    }

    createClass(SpotifyPlayer, [{
        key: 'fetchAlbumData',
        value: function fetchAlbumData() {
            var xhr = new XMLHttpRequest();
            var url = 'https://api.spotify.com/v1/search';
            var that = this;
            var queryString = '?q=album:' + this.state.albumName + ' artist:' + this.state.artistName;
            queryString += '&type=album';
            xhr.open('get', url + queryString);
            xhr.setRequestHeader('Authorization', 'Bearer ' + this.props.token);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200) {
                        var response = JSON.parse(xhr.responseText);
                        if (response.albums.items.length > 0) {
                            that.fetchTracks(response.albums.items[0].id, function (data) {
                                if (data.tracks.items.length > 0) {
                                    that.setState({ albumData: data });
                                } else {
                                    that.setState({ albumData: null, playerPlaceholder: that.props.noDataFoundText });
                                }
                            });
                        } else {
                            that.setState({ albumData: null, playerPlaceholder: that.props.noDataFoundText });
                        }
                    } else {
                        that.setState({ albumData: null, playerPlaceholder: that.props.noDataFoundText });
                    }
                }
            };
            xhr.send();
        }
    }, {
        key: 'fetchTracks',
        value: function fetchTracks(albumId, callback) {
            var xhr = new XMLHttpRequest();
            var that = this;
            xhr.open('get', 'https://api.spotify.com/v1/albums/' + albumId);
            xhr.setRequestHeader('Authorization', 'Bearer ' + this.props.token);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200) {
                        var response = JSON.parse(xhr.responseText);
                        callback(response);
                    } else {
                        that.setState({ albumData: null, playerPlaceholder: that.props.noDataFoundText });
                    }
                }
            };
            xhr.send();
        }
    }, {
        key: 'firePlayingStatusChangeEvent',
        value: function firePlayingStatusChangeEvent(isPlaying, audioTrack, spotifyTrack) {
            if (isPlaying && this.props.onTrackPlayed) {
                this.props.onTrackPlayed(audioTrack, spotifyTrack);
            } else if (!isPlaying && this.props.onTrackPaused) {
                this.props.onTrackPaused(audioTrack, spotifyTrack);
            }
        }
    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            if (this.props.token === null) {
                console.error('A valid access token is needed to access spotify API');
                return;
            }
            this.fetchAlbumData();
        }
    }, {
        key: 'componentWillUpdate',
        value: function componentWillUpdate(newProps) {
            if (newProps.albumName !== this.props.albumName || newProps.artistName !== this.props.artistName) {
                this.setState({ artistName: newProps.artistName, albumName: newProps.albumName }, this.fetchAlbumData);
            }
        }
    }, {
        key: 'render',
        value: function render() {
            var playlistHeader = function () {
                if (this.props.showHeader) {
                    return React.createElement(
                        'div',
                        { className: 'media' },
                        React.createElement(
                            'div',
                            { className: 'media-left' },
                            React.createElement(
                                'a',
                                { href: this.state.albumData.external_urls.spotify },
                                React.createElement('img', {
                                    alt: 'cover image',
                                    className: 'media-object',
                                    height: '64',
                                    src: this.state.albumData.images[0].url,
                                    width: '64'
                                })
                            )
                        ),
                        React.createElement(
                            'div',
                            { className: 'media-body' },
                            React.createElement(
                                'h4',
                                { className: 'media-heading' },
                                this.state.albumData.name
                            ),
                            this.state.albumData.release_date
                        )
                    );
                }
                return null;
            }.bind(this);
            if (this.state.albumData === null) {
                return React.createElement(
                    'section',
                    { className: 'spotify-player' },
                    React.createElement('div', { dangerouslySetInnerHTML: { __html: this.state.playerPlaceholder } })
                );
            } else {
                return React.createElement(
                    'section',
                    { className: 'spotify-player' },
                    playlistHeader(),
                    React.createElement(TrackList, {
                        listGroupClassName: this.props.listGroupClassName,
                        listGroupItemBadgeClassName: this.props.listGroupItemBadgeClassName,
                        listGroupItemClassName: this.props.listGroupItemClassName,
                        listGroupItemLink: this.props.listGroupItemLink,
                        previewWarningText: this.props.previewWarningText,
                        tracks: this.state.albumData.tracks.items,
                        updateTrackPlayingStatus: this.firePlayingStatusChangeEvent
                    })
                );
            }
        }
    }]);
    return SpotifyPlayer;
}(React.Component);

SpotifyPlayer.propTypes = {
    albumName: PropTypes.string.isRequired,
    artistName: PropTypes.string.isRequired,
    listGroupClassName: PropTypes.string,
    listGroupItemBadgeClassName: PropTypes.string,
    listGroupItemClassName: PropTypes.string,
    listGroupItemLink: PropTypes.string,
    noDataFoundText: PropTypes.string,
    onTrackPaused: PropTypes.func,
    onTrackPlayed: PropTypes.func,
    previewWarningText: PropTypes.string,
    showHeader: PropTypes.bool,
    token: PropTypes.string
};
SpotifyPlayer.defaultProps = {
    listGroupClassName: 'list-group',
    listGroupItemBadgeClassName: 'badge',
    listGroupItemClassName: 'list-grouprequired to work-item',
    listGroupItemLink: '',
    noDataFoundText: 'No data found',
    onTrackPaused: undefined,
    onTrackPlayed: undefined,
    previewWarningText: 'Only 30 seconds preview',
    showHeader: true,
    token: null //https://developer.spotify.com/web-api/
};

// module.exports = require('./SpotifyPlayer');
// export { Track } from "./Track";
// export { TrackList } from "./TrackList";

exports.SpotifyPlayer = SpotifyPlayer;
//# sourceMappingURL=index.js.map
